@@ -196,8 +196,32 @@ def solve_linear_system(x1, y1, x2, y2):
 	b = y1 - m*x1
 	return m, b
 
+
 # %% ============================= array check =========================== %% #
-def index(x, value, closest=True):
+# def index(x, value, closest=True):
+#     """Returns the first index of the element in array.
+
+#     Args:
+#         x (list or array): 1D array.
+#         value (float or int): value.
+#         closest (book, optional): if True, returns the index of the element in 
+#             array which is closest to value.
+
+#     Returns:
+#         index (int)
+#     """
+#     # backpack developers note!!!!
+#     # if this function changes, it needs to be copied to these files: figmanip
+
+#     if closest:
+#         # return int(np.argmin(np.abs(  np.array(x)-value)   ))
+#         _inner1 = np.array(x) - value
+#         _inner2 = np.ma.masked_array(_inner1, np.isnan(_inner1))
+#         return int(np.argmin(np.abs(_inner2)))
+#     else:
+#         return np.where(x == value)[0]
+
+def index(x, value, closest=True, roundup=False):
     """Returns the first index of the element in array.
 
     Args:
@@ -205,18 +229,34 @@ def index(x, value, closest=True):
         value (float or int): value.
         closest (book, optional): if True, returns the index of the element in 
             array which is closest to value.
+        roundup (bool, optional): if closest=True, and value is exactly midway
+            between 2 items in array x, rounup=True will return the index of 
+            item in x with highest value. Default is False.
 
     Returns:
         index (int)
     """
     # backpack developers note!!!!
     # if this function changes, it needs to be copied to these files: figmanip
-
     if closest:
-        # return int(np.argmin(np.abs(  np.array(x)-value)   ))
         _inner1 = np.array(x) - value
         _inner2 = np.ma.masked_array(_inner1, np.isnan(_inner1))
-        return int(np.argmin(np.abs(_inner2)))
+        absv    = np.abs(_inner2)
+        vmin    = np.min(absv)
+        if np.sum(np.where(absv==vmin, 1, 0)) > 1:
+            indexes = [_[0] for _ in np.argwhere(absv==vmin)]
+            if roundup:
+                if x[indexes[0]] > x[indexes[1]]:
+                    return indexes[0]
+                else:
+                    return indexes[1]
+            else:
+                if x[indexes[0]] < x[indexes[1]]:
+                    return indexes[0]
+                else:
+                    return indexes[1]
+        else:
+            return int(np.argmin(np.abs(_inner2)))
     else:
         return np.where(x == value)[0]
 
@@ -465,6 +505,84 @@ def flatten(x):
     if len(x) == 0:
         return x
 
-    if len(np.array(x).shape) == 1:
-        return x
-    return np.concatenate(x).ravel()
\ No newline at end of file
+    # if len(np.array(x).shape) == 1:
+    #     return x
+    return np.concatenate(x).ravel()
+
+def digitize(x, bins, xmin=None, xmax=None):
+    """Returns dictionary with bin values (keys) and indexes. Wrapper for np.digitize and np.histogram_bin_edges)
+    
+    bin values is the central value between two bin edges and the indexes 
+    indicate the indexes of x items that goes in that bin
+
+    Args:
+        x (list): list with values to place in the bins.
+        bins (int, str, list): If bins is a list, it is assumed to be the bins edges. Note that,           
+            bins is inclusive at the bin_edge with smaller value, 
+            and exclusive at the bin_edge with higher value.
+        
+            If bins is an int (from np.histogram_bin_edges), it defines the number of equal-width 
+            bins in the given range (10, by default). If bins is a sequence, it defines the 
+            bin edges, including the rightmost edge, allowing for non-uniform bin widths.
+
+            If bins is a string from the list below (from np.histogram_bin_edges), 
+            histogram_bin_edges will use the method 
+            chosen to calculate the optimal bin width and consequently the number of bins 
+            (see the Notes section for more detail on the estimators) from the data that 
+            falls within the requested range. While the bin width will be optimal for the 
+            actual data in the range, the number of bins will be computed to fill the entire 
+            range, including the empty portions. For visualization, using the 'auto' option 
+            is suggested. Weighted data is not supported for automated bin size selection.
+
+            'auto'
+            Minimum bin width between the 'sturges' and 'fd' estimators. Provides good 
+            all-around performance.
+
+            'fd' (Freedman Diaconis Estimator)
+            Robust (resilient to outliers) estimator that takes into account data variability 
+            and data size.
+
+            'doane'
+            An improved version of Sturges' estimator that works better with non-normal datasets.
+
+            'scott'
+            Less robust estimator that takes into account data variability and data size.
+
+            'stone'
+            Estimator based on leave-one-out cross-validation estimate of the integrated squared error. 
+            Can be regarded as a generalization of Scott's rule.
+
+            'rice'
+            Estimator does not take variability into account, only data size. Commonly overestimates 
+            number of bins required.
+
+            'sturges'
+            R's default method, only accounts for data size. Only optimal for gaussian data and 
+            underestimates number of bins for large non-gaussian datasets.
+
+            'sqrt'
+            Square root (of data size) estimator, used by Excel and other programs for its speed and simplicity.
+        xmin, xmax (int, optional): minimum/maximum value for bins (only used if bins are not explicitly 
+            given. Default is None. If None, min and max array values are used. 
+            
+        Returns:
+            indexes (dict), bin_edges (list)
+    """
+    if isinstance(bins, Iterable)==False or isinstance(bins, str):
+        if xmin is None: xmin = np.min(x)
+        if xmax is None: xmax = np.max(x)
+        assert xmax > xmin, 'xmax must be higher than xmin'
+        bin_edges = np.histogram_bin_edges(x, bins=bins, range=(xmin, xmax), weights=None)
+    else:
+        assert xmin is None, 'if bins is a list (assumed to be the bins edges), than xmin must be None'
+        assert xmax is None, 'if bins is a list (assumed to be the bins edges), than xmax must be None'
+        bin_edges = bins
+    temp = np.digitize(x, bins=bin_edges, right=False)
+    
+    # building indexes dictionary
+    indexes = {}
+    for i in range(len(bin_edges[:-1])):
+        # indexes[(bin_edges[i]+bin_edges[i+1])/2] = [_ for _ in np.argwhere(temp==i+1)]
+        indexes[(bin_edges[i]+bin_edges[i+1])/2] = [int(_[0]) for _ in list(np.argwhere(temp==i+1))]
+        
+    return indexes, bin_edges
