@@ -3,6 +3,7 @@
 """Core functions for ID32 beamline at ESRF"""
 
 # %% ------------------------- Standard Imports --------------------------- %% #
+from collections.abc import Iterable
 from pathlib import Path
 import numpy as np
 import datetime
@@ -92,6 +93,9 @@ def _polarization(value):
     """returns polarization string based on undulator gap"""
     pol = value
 
+    if isinstance(value, Iterable):
+        value = np.mean(value)
+        
     if value < -5:
         pol = 'CM'
     elif -5 <= value < 5:
@@ -392,7 +396,14 @@ def read(TOP, sample, dataset, scan, branch=None, detectors_rixs_branch=['dbig_n
                         s.metadata[key] = initial['instrument']['positioners'][metadata[branch][key]][()]
                     except KeyError:
                         pass
-
+                
+                s.additional = {}
+                for key in additional[branch]:
+                    try:    
+                        s.additional[key] = initial['measurement'][additional[branch][key]][()]
+                    except KeyError:
+                        pass
+                    
                 return s
             else:
                 if branch == 'rixs': # ref indicates how to find the image number via the filename of the image
@@ -490,6 +501,7 @@ def read(TOP, sample, dataset, scan, branch=None, detectors_rixs_branch=['dbig_n
             _s.pol = pol
 
             _s.metadata = _metadata
+            _s.additional = _additional
 
         return out
 # %%
